function [vip, vipsgc, info, iters, allcomm] = commgauss(A,i,func,nnodes,opts)
%COMMGAUSS Communicability computation via Gauss quadrature.
%   node = COMMGAUSS(A,i) identifies the most important node, according to
%	communicability of node i, of a network whose adjacency matrix is A.
%
%   nodes = COMMGAUSS(A,i,func,nnodes,opts) adds more information.
%   func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info] = COMMGAUSS(A,i,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of communicability for each node of the list.
%	info: info(1)=0 means success, info(1)=1 means that a convergence
%	      test was not satisfied (see code). info(2) contains the number
%	      of matrix vector products performed during the computation.
%	iters: contains the number of iterations required for each node.
%	allsg: approximation of communicabilities of node i (unsorted).

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, reichel@math.kent.edu
%
%   Last revised Feb 3, 2016

if (nargin<5) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<4) || isempty(nnodes), nnodes = 1; end
if (nargin<3) || isempty(func), func = 'exp'; end
if (nargin<2), error('Too few input'); end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if strcmpi(func,'exp')
	flagexp = 1;
elseif strcmpi(func,'res')
	flagexp = 0;
else
	error('Incorrect function!')
end

if ~sum(A(:,i))
	error('NotCon:commgauss','Node %d is not connected',i)
end

nam = fieldnames(opts);
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)), opts.gausstolq = 1e-3; end
% max number of iterations
if ~any(strcmpi('gaussmaxn',nam)), opts.gaussmaxn = 200; end
% spectrum shift for gaussexp
if ~any(strcmpi('gaussmu',nam)), opts.gaussmu = eigs(double(A),1,'LA'); end
% alpha for resolvent
if ~any(strcmpi('alpha',nam)), opts.alpha = 0; end
% show = 1 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

if ~isempty(findobj('type','figure','name','vipnodes')), opts.show = 1; end
n = size(A,1);

allcomm = zeros(n,1);
iters = zeros(n,1);
y = zeros(n,1);
mvp = 0;
info = 0;

theta = opts.gaussmu;
if ~flagexp
	alpha = opts.alpha;
	if (alpha >= 1/theta) || (alpha<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',theta);
	end
end

sqt = sqrt(2);
y(i) = 1;
if opts.show, hwait = waitbar(0,'','name','Computing'); end
for j = 1:n
	if j == i
		if flagexp
			[allcomm(j), infog] = gaussexp(A,y,theta,opts);
		else
			[allcomm(j), infog] = gaussres(A,y,alpha,opts);
		end
		iters(j) = infog;
		mvp = mvp + infog;
	else
		y(j) = 1;
		if flagexp
			[allcomm1, infog] = gaussexp(A,y/sqt,theta,opts);
		else
			[allcomm1, infog] = gaussres(A,y/sqt,alpha,opts);
		end
		iters(j) = infog;
		mvp = mvp + infog;
		y(j) = -1;
		if flagexp
			[allcomm2, infog] = gaussexp(A,y/sqt,theta,opts);
		else
			[allcomm2, infog] = gaussres(A,y/sqt,alpha,opts);
		end
		iters(j) = iters(j) + infog;
		mvp = mvp + infog;
		allcomm(j) = 1/2*(allcomm1 - allcomm2);
		y(j) = 0;
	end
	if opts.show
		if isvalid(hwait)
			waitbar(j/n,hwait);
		else
			error('calcola:stopped',...
				'Computation terminated by user')
		end
	end
end
if opts.show, close(hwait); end

if any(iters) >= opts.gaussmaxn
	info = 1;
end

info = [info mvp];

[vipcomm, vip] = sort(allcomm,'descend');

vip = vip(1:nnodes);
vipsgc = vipcomm(1:nnodes);

if opts.show == 10
	if isempty(findobj('type','figure','name','vipnodes'))
		fprintf('%d VIP node',nnodes)
		if nnodes > 1
			fprintf('s identified:\n')
		else
			fprintf(' identified:\n')
		end
		fprintf('%8d (%.2e)\n',[vip vipsgc]')
	end
end

