function [vip,vipsgc,info,viplr,vipsglr,S,F] = commhyb(A,i,func,nnodes,opts,viplr,vipsglr,infolr)
%COMMHYB Communicability computation via hybrid algorithm.
%   node = COMMHYB(A,i) identifies the most important node, according to
%   communicabilities of node i, of a network whose adjacency matrix is A.
%
%   nodes = COMMHYB(A,i,func,nnodes,opts) adds more information.
%   func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info] = COMMHYB(A,i,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of communicabilities between node i and
%	      each node of the list.
%	info: info(1)=0 means success, info(1)=1 means that the weak
%	      convergence test was satisfied (see code), info(1)=2 means
%	      failure. info(2) contains the number of matrix vector
%	      products performed during the computation.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   David Martin & Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it,
%          dmarti49@kent.edu, reichel@math.kent.edu
%
%   Last revised Aug 26, 2015

if (nargin<5) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<4) || isempty(nnodes), nnodes = 1; end
if (nargin<3) || isempty(func), func = 'exp'; end
if (nargin<2), error('Too few input arguments'), end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if strcmpi(func,'exp')
	flagexp = 1;
elseif strcmpi(func,'res')
	flagexp = 0;
else
	error('Incorrect function!')
end

if ~sum(A(:,i))
	error('NotCon:commgauss','Node %d is not connected',i)
end

nam = fieldnames(opts);
% tolerance for weak convergence test in sgcenlr
if ~any(strcmpi('sgctoll',nam)), opts.sgctoll = 1e-3; end
% alpha for resolvent
if ~any(strcmpi('alpha',nam)), opts.alpha = 0; end
% show = 1 slow, show = 2 fast, show = 10 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

n = size(A,1);


opts.sgcshift = 0;
if nargin < 6
	[viplr, vipsglr, infolr, S, F, opts.gaussmu] = commlr(A,i,func,nnodes,opts);
end

info = infolr(1);	% 0 strong conv, 1 weak conv, 2 failure
mvp = infolr(3);

if opts.show == 10
	fprintf('\nStarting Gauss quadrature computation!\n')
end

sqt = sqrt(2);
y = zeros(n,1);
y(i) = 1;
siz = size(viplr,1);
allcomm = zeros(siz,1);
iters = zeros(siz,1);

theta = opts.gaussmu;
if ~flagexp
	alpha = opts.alpha;
	if (alpha >= 1/theta) || (alpha<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',theta);
	end
end

opts.gaussmu = 0;
for j = 1:siz
	if viplr(j) == i
		if flagexp
			[allcomm(j), infog] = gaussexp(A,y,theta,opts);
		else
			[allcomm(j), infog] = gaussres(A,y,alpha,opts);
		end
		iters(j) = infog;
		mvp = mvp + infog;
	else
		y(viplr(j)) = 1;
		if flagexp
			[allcomm1, infog] = gaussexp(A,y/sqt,theta,opts);
		else
			[allcomm1, infog] = gaussres(A,y/sqt,alpha,opts);
		end
		iters(j) = infog;
		mvp = mvp + infog;
		y(viplr(j)) = -1;
		if flagexp
			[allcomm2, infog] = gaussexp(A,y/sqt,theta,opts);
		else
			[allcomm2, infog] = gaussres(A,y/sqt,alpha,opts);
		end
		iters(j) = iters(j) + infog;
		mvp = mvp + infog;
		allcomm(j) = 1/2*(allcomm1 - allcomm2);
		%keyboard
		y(viplr(j)) = 0;
	end
end

[~, isgs] = sort(allcomm,'descend');
vip = viplr(isgs);
vipsgc = allcomm(isgs);
vip = vip(1:nnodes);
vipsgc = vipsgc(1:nnodes);

info = [info mvp];

if opts.show == 10
	if isempty(findobj('type','figure','name','vipnodes'))
		fprintf('%d VIP node',nnodes)
		if nnodes > 1
			fprintf('s identified:\n')
		else
			fprintf(' identified:\n')
		end
		fprintf('%8d (%.2e)\n',[vip vipsgc]')
	end
end

