function [fval, info] = gaussexp(A,u,theta,opts)
%GAUSSEXP Compute u'*expm(A)*u via Gauss quadrature.
%   fval = GAUSSEXP(A,u,theta) computes u'*expm(A)*u, for a matrix A and
%   a column vector u, using Gauss quadrature. The parameter theta
%   should contain an estimate of the largest eigenvalue of A.
%
%   fval = GAUSSEXP(A,u,theta,opts) adds more information.
%	opts is a structure with fields:
%       gausstolq: tolerance for quadrature convergence (default 1e-3)
%       gausstau: tolerance to detect a breakdown in the Lanczos algorithm (default 1e-10)
%       gaussmaxn: max number of iterations (default 100)
%       gaussmu: value of mu for scaling expm (defaults to theta)
%       gaussreorth: reorthormalization of the columns of the Krylov matrix
%                                             0 no reorth 
%                                             1 modified Gram-Schmidt reorth
%                                             2 classical Gram-Schmidt reorth
%                                             3 double classical Gram-Schmidt reorth
%       gaussmgs: computes the new Lanczos vector by usign a modified Gram-Schmidt reorth
%
%   [fval info] = GAUSSEXP(A,...) gets more information.
%	info: number of matrix vector products performed during the
%	      computation.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, reichel@math.kent.edu
%
%   Last revised Jan 15, 2016

if nargin < 4 || isempty(opts), opts = struct('empty','empty'); end
if nargin < 3, error('Too few input arguments.'); end

m = size(u,1);
if isnumeric(A)
	if ~isequal(A,A'), error('A must be symmetric.'), end
	mvprod = @(x) A*x;
elseif isa(A,'char') || isa(A,'function_handle') || isa(A,'inline')
	mvprod = A;
else
	error('A must be either a numeric matrix or a function.')
end

nam = fieldnames(opts);
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)), opts.gausstolq = 1e-3; end
% tolerance for check if beta is 0
if ~any(strcmpi('gausstau',nam)), opts.gausstau = 1e-10; end
% max number of iterations
if ~any(strcmpi('gaussmaxn',nam)), opts.gaussmaxn = min(100,m); end
% value of mu for scaling expm, defaults to theta
if ~any(strcmpi('gaussmu',nam)), opts.gaussmu = theta; end
% 1 for reorth (see code)
if ~any(strcmpi('gaussreorth',nam)), opts.gaussreorth = 0; end
% 1 for mgs
if ~any(strcmpi('gaussmgs',nam)), opts.gaussmgs = 1; end

mu = opts.gaussmu;

expm_fun = @(T,mu) expm_eig(T,mu);

beta = norm(u);

sqrteps = sqrt(eps);

if opts.gaussreorth, Q = zeros(m,opts.gaussmaxn+1); end
T = zeros(opts.gaussmaxn+1);
alf = zeros(opts.gaussmaxn,1);
bet = [beta;zeros(opts.gaussmaxn,1)];

q0 = zeros(m,1);
q = u/beta;
if opts.gaussreorth, Q(:,1) = q; end

dif = 1000;
g = 0;
i = 0;

while (dif > opts.gausstolq*abs(g)) && (i<opts.gaussmaxn)
	
	i = i + 1;
	
	v = mvprod(q);
	if opts.gaussmgs
		v = v - beta*q0;
		alfa = q'*v;
		v = v - alfa*q;
	else
		alfa = q'*v;
		v = v - beta*q0 - alfa*q;
	end
	
	% Reorthonormalization Columns of Q
	if opts.gaussreorth
		switch opts.gaussreorth
			case 1	% MGS
				for j = 1:i
					v = v - (v'*Q(:,j))*Q(:,j);
				end
			case 2	% CGS
				v = v - Q(:,1:i)*(Q(:,1:i)'*v);
			case 3	% double CGS
				c = Q(:,1:i)'*v;
				v = v - Q(:,1:i)*c;
				%alfa = alfa + sum(c);		% See irbleigs
				if norm(c) > sqrteps
					c1 = Q(:,1:i)'*v;
					v = v - Q(:,1:i)*c1;
					%alfa = alfa + sum(c1);	% See irbleigs
				end
		end
	end
	
	beta = norm(v);
	q0 = q;
	
	if beta < opts.gausstau
		T(i,i) = alfa;
		S = expm_fun(T(1:i,1:i),mu);
		g = S(1,1);
		gr = g;
		break
	else
		q = v/beta;
	end
	
	if opts.gaussreorth, Q(:,i+1) = q; end
	
	alf(i) = alfa;
	bet(i+1) = beta;
	%[alfa beta]
	
	[pm, pm1] = opolyval(alf(1:i),bet(2:i+1),1,theta);
	alfk1 = theta - bet(i+1)*(pm1/pm);
	
	%T = diag([alf(1:i);alfk1]) + diag(bet(2:i+1),1) + diag(bet(2:i+1),-1);
	T(i:i+1,i:i+1) = [alfa beta;beta alfk1];
	S = expm_fun(T(1:i,1:i),mu);
	g = S(1,1);
	
	S = expm_fun(T(1:i+1,1:i+1),mu);
	gr = S(1,1);
	
	dif = abs(gr-g)/2;
end

fval = bet(1)^2 * (gr + g)/2;
info = i;

e = exp(mu);
if isfinite(e)		% recover correct values from spectral shift
	fval = e*fval;
else
	warning('gausstrexp:scaledCentrality',...
		'The result is scaled by exp(%.2e).',mu)
end


function F = expm_eig(T,mu)

if nargin < 2, mu = 0; end

[V, D] = eig(T);
F = V*diag(exp(diag(D)-mu))*V';


function [p, p1] = opolyval(alfa,beta,beta0,x)
%OPOLYVAL computation of orthonormal polynomials
%   p=opolyval(a,b,beta0,x) computes p_{k+1}(x), where x is a vector and
%   p_{k+1} is the orhonomal polynomial defined by the recursion
%      b_k*p_{k+1}(x) = (x-alpha_k)*p_k(x) - b_{k-1}*p_{k-1}(x),
%   with p_0(x)=1/beta0 and p_{-1}(x)=0; beta0 is the norm of p(x)=1.
%   The coefficients a(1:k) and b(1:k) of the orthonormal polynomials can be
%   obtained by the coefficients rho(1:k) and gamma(1:k) of the monic
%   orthogonal polynomials, by taking a_i=rho_i and b_i=sqrt(gamma_i).
%   [p,p1]=opolyval(...) returns also the values of p_k(x).

x = x(:);

p1 = ones(size(x,1),1)/beta0;
p = (x-alfa(1))/beta(1);

for k = 2:size(alfa,1)
	p0 = p1;
	p1 = p;
	p = (x-alfa(k)).*p1 - beta(k-1).*p0;
	p = p/beta(k);
end

