function [vip,vipsgc,info,viplr,vipsglr,S,F] = sgcenhyb(A,func,nnodes,opts,viplr,vipsglr,infolr)
%SGCENHYB Subgraph centrality computation via hybrid algorithm.
%   node = SGCENHYB(A) identifies the most important node, according to
%   subgraph centrality, of a network whose adjacency matrix is A.
%
%   nodes = SGCENHYB(A,func,nnodes,opts) adds more information.
%   func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info] = SGCENHYB(A,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of subgraph centrality for each node of the list.
%	info: info(1)=0 means success, info(1)=1 means that the weak
%	      convergence test was satisfied (see code), info(1)=2 means
%	      failure. info(2) contains the number of matrix vector
%	      products performed during the computation.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, reichel@math.kent.edu
%
%   Last revised Feb 9, 2016

if (nargin<4) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<3) || isempty(nnodes), nnodes = 1; end
if (nargin<2) || isempty(func), func = 'exp'; end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if strcmpi(func,'exp')
	flagexp = 1;
elseif strcmpi(func,'res')
	flagexp = 0;
else
	error('Incorrect function!')
end

nam = fieldnames(opts);
% tolerance for weak convergence test in sgcenlr
if ~any(strcmpi('sgctoll',nam)), opts.sgctoll = 1e-3; end
% alpha for resolvent
if ~any(strcmpi('alpha',nam)), opts.alpha = 0; end
% show = 1 slow, show = 2 fast, show = 10 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

n = size(A,1);

if nargin < 5
	[viplr, vipsglr, infolr, S, F, opts.gaussmu] = sgcenlr(A,func,nnodes,opts);
end

if opts.show == 10
	fprintf('\nStarting Gauss quadrature computation!\n')
end

info = infolr(1);	% 0 strong conv, 1 weak conv, 2 failure
mvp = infolr(3);

ei = zeros(n,1);
siz = size(viplr,1);
allsg = zeros(siz,1);
iters = zeros(siz,1);

theta = opts.gaussmu;
if ~flagexp
	alpha = opts.alpha;
	if (alpha >= 1/theta) || (alpha<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',theta);
	end
end

for i = 1:siz
	ei(viplr(i)) = 1;
	if flagexp
		[allsg(i), infog] = gaussexp(A,ei,theta,opts);
	else
		[allsg(i), infog] = gaussres(A,ei,alpha,opts);
	end
	iters(i) = infog;
	mvp = mvp + infog;
	ei(viplr(i)) = 0;
end

[~, isgs] = sort(allsg,'descend');
vip = viplr(isgs);
vipsgc = allsg(isgs);
vip = vip(1:nnodes);
vipsgc = vipsgc(1:nnodes);

info = [info mvp];

if opts.show == 10
	if isempty(findobj('type','figure','name','vipnodes'))
		fprintf('%d VIP node',nnodes)
		if nnodes > 1
			fprintf('s identified:\n')
		else
			fprintf(' identified:\n')
		end
		fprintf('%8d (%.2e)\n',[vip vipsgc]')
	end
end

