function [vip, vipsgc, info, iters, allstconv] = stconvgauss(A,func,nnodes,opts)
%STCONVGAUSS Starting convenience computation via Gauss quadrature.
%   node = STCONVGAUSS(A) identifies the most important node, according to
%	starting convenience, of a network whose adjacency matrix is A.
%
%   nodes = STCONVGAUSS(A,func,nnodes,opts) adds more information.
%   func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info] = STCONVGAUSS(A,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of starting convenience for each node of the
%	      list.
%	info: info(1)=0 means success, info(1)=1 means that a convergence
%	      test was not satisfied (see code). info(2) contains the number
%	      of matrix vector products performed during the computation.
%	iters: contains the number of iterations required for each node.
%	allstconv: approximation of starting convenience for each node
%	      (unsorted).

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, reichel@math.kent.edu
%
%   Last revised Feb 5, 2016

if (nargin<4) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<3) || isempty(nnodes), nnodes = 1; end
if (nargin<2) || isempty(func), func = 'exp'; end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if strcmpi(func,'exp')
	flagexp = 1;
elseif strcmpi(func,'res')
	flagexp = 0;
else
	error('Incorrect function!')
end

nam = fieldnames(opts);
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)), opts.gausstolq = 1e-3; end
% max number of iterations
if ~any(strcmpi('gaussmaxn',nam)), opts.gaussmaxn = 200; end
% spectrum shift for gaussexp
if ~any(strcmpi('gaussmu',nam)), opts.gaussmu = eigs(double(A),1,'LA'); end
% alpha for resolvent
if ~any(strcmpi('alpha',nam)), opts.alpha = 0; end
% show = 1 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

if ~isempty(findobj('type','figure','name','vipnodes')), opts.show = 1; end
n = size(A,1);

allstconv = zeros(n,1);
iters = zeros(n,1);
y = ones(n,1);
mvp = 0;
info = 0;

theta = opts.gaussmu;
if ~flagexp
	alpha = opts.alpha;
	if (alpha >= 1/theta) || (alpha<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',theta);
	end
end

np3 = n+3;  nm1 = n-1;

if opts.show, hwait = waitbar(0,'','name','Computing'); end
for i = 1:n
	y(i) = 2;
	if flagexp
		[allstconv1, infog] = gaussexp(A,y/(sqrt(np3)),theta,opts);
	else
		[allstconv1, infog] = gaussres(A,y/(sqrt(np3)),alpha,opts);
	end
	iters(i) = infog;
	mvp = mvp + infog;
	y(i) = 0;
	if flagexp
		[allstconv2, infog] = gaussexp(A,y/(sqrt(nm1)),theta,opts);
	else
		[allstconv2, infog] = gaussres(A,y/(sqrt(nm1)),alpha,opts);
	end
	iters(i) = iters(i) + infog;
	mvp = mvp + infog;
	allstconv(i) = (np3*allstconv1 - nm1*allstconv2)/4;
	y(i) = 1;
	if opts.show
		if isvalid(hwait)
			waitbar(i/n,hwait);
		else
			error('calcola:stopped',...
				'Computation terminated by user')
		end
	end
end
if opts.show, close(hwait); end

if any(iters) >= opts.gaussmaxn
	info = 1;
end
info = [info mvp];

[vipstconv, vip] = sort(allstconv,'descend');

vip = vip(1:nnodes);
vipsgc = vipstconv(1:nnodes);

if opts.show == 10
	if isempty(findobj('type','figure','name','vipnodes'))
		fprintf('%d VIP node',nnodes)
		if nnodes > 1
			fprintf('s identified:\n')
		else
			fprintf(' identified:\n')
		end
		fprintf('%8d (%.2e)\n',[vip vipsgc]')
	end
end

