function [vip,vipsgc,info,viplr,vipsglr,S,F] = stconvhyb(A,func,nnodes,opts,viplr,vipsglr,infolr)
%STCONVHYB Starting convenience computation via hybrid algorithm.
%   node = STCONVHYB(A) identifies the most important node, according to
%   starting convenience, of a network whose adjacency matrix is A.
%
%   nodes = STCONVHYB(A,func,nnodes,opts) adds more information.
%   func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info] = STCONVHYB(A,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of starting convenience for each node of the list.
%	info: info(1)=0 means success, info(1)=1 means that the weak
%	      convergence test was satisfied (see code), info(1)=2 means
%	      failure. info(2) contains the number of matrix vector
%	      products performed during the computation.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   David Martin & Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it,
%          dmarti49@kent.edu, reichel@math.kent.edu
%
%   Last revised Aug 27, 2015

if (nargin<4) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<3) || isempty(nnodes), nnodes = 1; end
if (nargin<2) || isempty(func), func = 'exp'; end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if strcmpi(func,'exp')
	flagexp = 1;
elseif strcmpi(func,'res')
	flagexp = 0;
else
	error('Incorrect function!')
end

nam = fieldnames(opts);
% additional nodes for weak convergence test
if ~any(strcmpi('sgcincr',nam)), opts.sgcincr = 5; end
% tolerance for weak convergence test in sgcenlr
if ~any(strcmpi('sgctoll',nam)), opts.sgctoll = 1e-3; end
% max number of iterations
if ~any(strcmpi('sgcnmax',nam)), opts.sgcnmax = 100; end
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)), opts.gausstolq = 1e-3; end
% show = 1 slow, show = 2 fast, show = 10 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

n = size(A,1);

if nargin < 5
	[viplr, vipsglr, infolr, S, F, opts.gaussmu] = stconvlr(A,func,nnodes,opts);
end

info = infolr(1);	% 0 strong conv, 1 weak conv, 2 failure
mvp = infolr(3);

if opts.show == 10
	fprintf('\nStarting Gauss quadrature computation!\n')
end

siz = size(viplr,1);
allstconv = zeros(siz,1);
iters = zeros(siz,1);

np3 = n+3;  nm1 = n-1;
y = ones(n,1);

theta = opts.gaussmu;
if ~flagexp
	alpha = opts.alpha;
	if (alpha >= 1/theta) || (alpha<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',theta);
	end
end

for i = 1:siz
	y(viplr(i)) = 2;
	if flagexp
		[allstconv1, infog] = gaussexp(A,y/sqrt(np3),theta,opts);
	else
		[allstconv1, infog] = gaussres(A,y/sqrt(np3),alpha,opts);
	end
	iters(i) = infog;
	mvp = mvp + infog;
	y(viplr(i)) = 0;
	if flagexp
		[allstconv2, infog] = gaussexp(A,y/sqrt(nm1),theta,opts);
	else
		[allstconv2, infog] = gaussres(A,y/sqrt(nm1),alpha,opts);
	end
	iters(i) = iters(i) + infog;
	mvp = mvp + infog;
	allstconv(i) = (np3*allstconv1 - nm1*allstconv2)/4;
	y(viplr(i)) = 1;
end

[~, isgs] = sort(allstconv,'descend');
vip = viplr(isgs);
vipsgc = allstconv(isgs);
vip = vip(1:nnodes);
vipsgc = vipsgc(1:nnodes);

info = [info mvp];

if opts.show == 10
	if isempty(findobj('type','figure','name','vipnodes'))
		fprintf('%d VIP node',nnodes)
		if nnodes > 1
			fprintf('s identified:\n')
		else
			fprintf(' identified:\n')
		end
		fprintf('%8d (%.2e)\n',[vip vipsgc]')
	end
end

