function [vip, vipsgc, info, S, F, musave, MOV] = stconvlr(A,func,nnodes,opts)
%STCONVLR Starting convenience computation via low-rank approximation.
%   node = STCONVLR(A) identifies the most important node, according to
%   starting convenience, of a network whose adjacency matrix is A.
%
%   nodes = STCONVLR(A,func,nnodes,opts) adds more information.
%	func: matrix function to be used, 'exp' or 'res' (default 'exp').
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%
%   [nodes values info S F mu] = STCONVNLR(A,...) gets more information.
%	nodes: list of nodes.
%	values: approximation of starting convenience for each node of the list.
%	info: info(1)=0 means success, info(1)=1 means that the weak
%	      convergence test was satisfied (see code), info(1)=2 means
%	      failure. info(2:end) from IRBLEIGS.
%	S: cell array containing all the lists of nodes generated.
%	F: approximation of starting convenience of each node.
%	mu: largest eigenvalue of A.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, reichel@math.kent.edu
%
%   Last revised Jan 14, 2016

if (nargin<4) || isempty(opts), opts = struct('empty','empty'); end
if (nargin<3) || isempty(nnodes), nnodes = 1; end
if (nargin<2) || isempty(func), func = 'exp'; end

if ~isnumeric(A), error('A must be a numeric matrix.'), end
if ~isequal(A,A'), error('A must be symmetric.'), end

if ~exist('irbleigs','file')
	error('The IRBLEIGS function is not in the search path.')
end

%	check options
nam = fieldnames(opts);
% max number of iterations
if ~any(strcmpi('sgcmaxn',nam)), opts.sgcmaxn = 300; end
% max number of columns for VA
if ~any(strcmpi('sgcmaxva',nam)), opts.sgcmaxva = 200; end
% additional nodes for weak convergence test
%if ~any(strcmpi('sgcincr',nam)), opts.sgcincr = 0; end
% tolerance for weak convergence test
if ~any(strcmpi('sgctoll',nam)), opts.sgctoll = 0; end
% tau
if ~any(strcmpi('sgctau',nam)), opts.sgctau = 1e-1; end
% min number of incr
if ~any(strcmpi('sgcminkappa',nam)), opts.sgcminkappa = 5; end
% max number of incr
if ~any(strcmpi('sgcmaxkappa',nam)), opts.sgcmaxkappa = 100; end
% sgcshift = 1 for spectrum shift, 0 otherwise
if ~any(strcmpi('sgcshift',nam)), opts.sgcshift = 1; end
% alpha for resolvent
if ~any(strcmpi('alpha',nam)), opts.alpha = 0; end
% show = 1 slow, show = 2 fast, show = 10 messages, show = 0 does not show
if ~any(strcmpi('show',nam)), opts.show = 0; end

%	the following options are for irbleigs
% number of eigenvalues
if ~any(strcmpi('k',nam)), opts.k = 20; end
% blocksize
if ~any(strcmpi('blsz',nam)), opts.blsz = 5; end
% number of blocks
if ~any(strcmpi('nbls',nam)), opts.nbls = ceil(opts.k/opts.blsz+1.1) + 2; end
% 'LE', 'SE', numeric value
if ~any(strcmpi('sigma',nam)), opts.sigma = 'LE'; end
% show intermediate results
if ~any(strcmpi('dispr',nam)), opts.dispr = 0; end
% maximum number of iterations
if ~any(strcmpi('maxit',nam)), opts.maxit = 100; end
% tolerance used for convergence
if ~any(strcmpi('tol',nam)), opts.tol = 1e-3; end

% for internal use
restype = 1;
makemovie = 0; MOV = [];

n = size(A,1);
if nargout>3, keepS = 1; else keepS = 0; end
VA = [];	% eigenvectors database

[V, D, info] = irbleigs(A,opts);
infosave = info;
lam = diag(D);
[lam, vind] = sort(lam,'descend');	% are the eigenvalues already sorted?
V = V(:,vind);
numeig = opts.k;
tsol = zeros(n,1);	% originally, the true solution

musave = lam(1);
if strcmpi(func,'exp')
	fun = @(x,mu) exp(x-mu);
	if opts.sgcshift	% spectrum shift
		mu = musave;
	else
		mu = 0;
	end
elseif strcmpi(func,'res')
	fun = @(x,mu) 1./(1-mu*x);
	mu = opts.alpha;
	if (mu >= 1/musave) || (mu<=0)
		error('Invalid value for alpha! The spectral radius of A is %.2e',musave);
	end
else
	error('Incorrect function!')
end

h_an = findobj('type','figure','name','Animation');
if opts.show
	allam = lam;
	scrsz = get(0,'ScreenSize');
	if isempty(h_an)
		h_an = figure('name','Animation','NumberTitle','off',...
			'Position',[1 scrsz(4)/3 scrsz(3)/2 2*scrsz(4)/3]);
	end
	figure(h_an)
	h_spy = subplot(2,3,1);
	set(h_spy,'xtick',[],'ytick',[]);
	%set(h_spy,'fontsize',12)
	hold(h_spy,'on')
	spy(A)
	drawnow
	title(h_spy,'Adjacency matrix','fontsize',12)
	hold(h_spy,'off')
	figure(h_an)
	h_spec = subplot(2,3,4);
	set(gca,'fontsize',12)
	plot(h_spec,lam,'o-')
	%axis square
	axis tight
	grid
	title('Computed spectrum', 'fontsize',12)
	ylv = [min(tsol) max(tsol)+eps];
	wid = ylv(2)-ylv(1);
	ylv = [max(0,ylv(1)-wid/2) ylv(2)+wid/2];
	figure(h_an)
	h_big = subplot(2,3,[2 3 5 6]);
	if any(tsol)
		plot(h_big,tsol,'r')
	else
		plot(h_big,0);
	end
	xlim([1 n])
	ylim(ylv)
	set(h_big,'fontsize',12)
	ht = text(1,1,'');
else
	if ~isempty(h_an)
		close(h_an)
	end
end

F = zeros(n,1);
utildek = zeros(n,1);
wtildek = 0;
if keepS, S = cell(numeig,1); end

N = 0;
realN = 0;
restart = 0;
flag = 1;
lis = [];
while flag && (realN<min(opts.sgcmaxn,n)) && (N<numeig)
	N = N+1;
	realN = realN+1;
	f_lam = fun(lam(N),mu);
	v = V(:,N);
	sv = sum(v);
	utildek = utildek + v.^2;
	wtildek = wtildek + sv^2;
	contr = f_lam*v*sv;
	F = F + contr;
	G = f_lam*sqrt(1-utildek)*sqrt(n-wtildek);
	L = F - G;
	U = F + G;
	[Fs, indx] = sort(F,'descend');
	Fmax = Fs(nnodes);
	%Ls = sort(L,'descend');
	%Lmax = Ls(nnodes);		% Lmax = max(L);
	lis = find(U >= Fmax);
	if keepS, S{realN} = lis; end
	
	% exit when the nodes are identified or lower bounds stagnate
	flag = (numel(lis)>nnodes) && (abs(mean(contr))>opts.sgctoll*Fmax);
	
	if opts.show
		%fprintf('N=%d, Lmax=%.2e, #S=%d\n', realN, Lmax, numel(lis))
		if isvalid(h_an)
			figure(h_an)
		else
			error('calcola:stopped',...
				'Computation terminated by user')
		end
		h_big = subplot(2,3,[2 3 5 6]);
		hold(h_big,'on');
		if any(tsol)
			hh = plot(h_big,[1:n]',U,'b',[1:n]',L,'g',[1:n]',tsol,'r');
			set(hh(3),'linewidth',1.5)
			legend('upper bounds','lower bounds','true solution')
		else
			hh = plot(h_big,[1:n]',U,'b',[1:n]',L,'g');
			legend('upper bounds','lower bounds')
			axis tight
			ylv(1) = min(ylv(1),1.5*min(L));
			ylv(2) = max(ylv(2),1.5*max(U));
			ylim(ylv)
		end
		title(h_big,sprintf('N=%d - #S=%d\n', realN, numel(lis)), ...
			'fontsize',13,'units','normalized','position',[.5 .97 0])
		set(h_big,'xtick',[0:n:n],'ytick',[]);
		hold off
		drawnow
		if opts.show == 1
			pause(.2)
		end
		
		if makemovie
			delete(ht)
			ypos = .8 * (ylv(2)-ylv(1)) + ylv(1);
			ht = text(50,ypos,sprintf('N=%d - #S=%d\n', ...
				realN, numel(lis)), 'fontsize',16);
			MOV(N) = getframe;
		end
	end
	
	if N == numeig		% restart
		if isfield(opts,'eigvec'), opts = rmfield(opts,'eigvec'); end
		if isfield(opts,'sigma'), opts = rmfield(opts,'sigma'); end
		switch restype
			case 1
				VA = [VA V];
				if size(VA,2) >= n,
					break
				end
				opts.eigvec = VA;
				[V, D, info] = irbleigs(A,opts);
				infosave = infosave + info;
				lam = diag(D);
				[lam, vind] = sort(lam,'descend');
				V = V(:,vind);
				if keepS, S = [S; cell(numeig,1)]; end
				restart = restart + 1;
				N = 0;
				% switch to other restart if VA is too large
				if size(VA,2) >= opts.sgcmaxva,
					restype = 2;
				end
			case 2
				opts.sigma = lam(N);
				[V, D, info] = irbleigs(A,opts);
				infosave = infosave + info;
				lam = diag(D);
				[lam, vind] = sort(lam,'descend');
				V = V(:,vind);
				[~, ill] = min(abs(lam-opts.sigma));
				if keepS, S = [S; cell(numeig-ill,1)]; end
				restart = restart + 1;
				N = ill;
			otherwise
				error('wrong restart')
		end
		if opts.show
			allam = [allam; lam(N+1:numeig)];
			figure(h_an)
			%subplot(2,3,4)
			plot(h_spec,allam,'o-')
			set(gca,'fontsize',12)
			%axis square
			axis(h_spec,'tight')
			grid(h_spec,'on')
			%grid
			title(h_spec,'Computed spectrum', 'fontsize',12)
			drawnow
		end
	end
end

if keepS, S = S(1:realN); end

if flag
	warning('sgcenlr:tooFewEigenvales',...
		'Too few eigenvalues to identify VIP nodes.')
	info = [2 infosave];
	nvip = numel(lis);
else
	if numel(lis) > nnodes
		ivec = find((Fmax-Fs(nnodes+1:end))<opts.sgctau*Fmax);
		incr = min(length(ivec),opts.sgcmaxkappa);
		if incr < opts.sgcminkappa, incr = opts.sgcminkappa; end
		info = [1 infosave];
	else
		incr = 0;
		info = [0 infosave];
	end
	nvip = nnodes+incr;
end

vip = indx(1:nvip);
vipsgc = Fs(1:nvip);

if opts.show
	figure(h_an)
	h_big = subplot(2,3,[2 3 5 6]);
	hold(h_big,'on');
	hh2 = plot(1:n,U,'b',1:n,L,'g',vip,vipsgc,'r.','markersize',20);
	legend(hh2,'upper bounds','lower bounds','candidate nodes')
	hold off
end

if strcmpi(func,'exp')
	if isfinite(exp(mu))
		vipsgc = exp(mu)*vipsgc;
	else
		warning('sgcenlr:scaledCentrality',...
			'Subgraph centrality is scaled by exp(mu).')
	end
end

if opts.show == 10
	switch info(1)
		case 0
			fprintf('Strong convergence test was satisfied.\n')
		case 1
			fprintf('Weak convergence test was satisfied.\n')
		case 2
			fprintf('No convergence test was satisfied.\n')
	end
	fprintf('N=%d - %d restarts - %d VIP node',realN,restart,nvip)
	if nvip > 1
		fprintf('s identified:\n')
	else
		fprintf(' identified:\n')
	end
	fprintf('%8d (%.2e)\n',[vip vipsgc]')
end

if info(2)
	ss = sprintf('IRBLEIGS did not converge.');
	uiwait(warndlg(ss,'Warning','modal'));
end


